import picologging
import io

def test_basic_handler():
    handler = picologging.Handler()
    record = picologging.LogRecord('test', picologging.INFO, 'test', 1, 'test', (), None, None, None)
    handler.handle(record)

def test_stream_handler():
    stream = io.StringIO()
    handler = picologging.StreamHandler(stream)
    record = picologging.LogRecord('test', picologging.INFO, __file__, 1, 'test', (), None, None, None)
    formatter = picologging.Formatter('%(message)s')
    handler.setFormatter(formatter)
    assert handler.formatter == formatter
    handler.handle(record)
    assert stream.getvalue() == 'test\n'

def test_custom_handler():
    class CustomHandler(picologging.Handler):
        def __init__(self):
            super().__init__()
            self.records = []

        def emit(self, record):
            self.records.append(record)

    handler = CustomHandler()
    record = picologging.LogRecord('test', picologging.INFO, __file__, 1, 'test', (), None, None, None)
    formatter = picologging.Formatter('%(message)s')
    handler.setFormatter(formatter)
    handler.handle(record)
    assert len(handler.records) == 1
    assert handler.records[0] == record


def test_delete_handler():
    handler = picologging.Handler()
    del handler

def test_add_acquire_release():
    handler = picologging.Handler()
    handler.acquire()
    assert handler.release() is None

def test_init_with_name():
    handler = picologging.Handler(name='test')
    assert handler.name == 'test'

def test_init_with_level():
    handler = picologging.Handler(level=picologging.DEBUG)
    assert handler.level == picologging.DEBUG



import picologging

picologging.basicConfig()


def test_picologging_handler(capsys):
    logger = picologging.getLogger(__name__)
    try:
        1/0
    except ZeroDivisionError:
        logger.exception("bork")
    cap = capsys.readouterr()
    result = cap.err + cap.out
    assert "bork" in result
    assert "ZeroDivisionError: division by zero" in result



from picologging import PercentStyle, LogRecord, INFO
import pytest

def test_percentstyle():
    perc = PercentStyle("%(msg)s %(levelno)d %(name)s")
    record = LogRecord("test", INFO, __file__, 1, "hello", (), None, None, None)
    assert perc.format(record) == "hello 20 test"

def test_percentstyle_format_bad_argument():
    perc = PercentStyle("%(msg)s %(levelno)d %(name)s")
    with pytest.raises(AttributeError):
        perc.format(None) 
    with pytest.raises(AttributeError):
        perc.format("") 
    with pytest.raises(AttributeError):
        perc.format({})

def test_custom_attribute():
    perc = PercentStyle("%(custom)s")
    record = LogRecord("test", INFO, __file__, 1, "hello", (), None, None, None)
    record.custom = "custom"
    assert perc.format(record) == "custom"



import io
import picologging
import logging
import pytest

def test_logger_attributes():
    logger = picologging.Logger('test')
    assert logger.name == 'test'
    assert logger.level == logging.NOTSET
    assert logger.parent == None
    assert logger.propagate == True
    assert logger.handlers == []
    assert logger.disabled == False
    assert logger.propagate == True


levels = [
    logging.DEBUG,
    logging.INFO,
    logging.WARNING,
    logging.ERROR,
    logging.CRITICAL,
    logging.NOTSET,
]

@pytest.mark.parametrize("level", levels)
def test_logging_custom_level(level):
    logger = picologging.Logger('test', level)
    assert logger.level == level


def test_set_level():
    logger = picologging.Logger('test')
    logger.setLevel(logging.DEBUG)
    assert logger.level == logging.DEBUG


def test_get_effective_level():
    logger = picologging.Logger('test')
    parent = picologging.Logger('parent', logging.DEBUG)
    logger.parent = parent
    assert logger.getEffectiveLevel() == logging.DEBUG
    assert logger.level == logging.NOTSET
    logger.setLevel(logging.WARNING)
    assert logger.getEffectiveLevel() == logging.WARNING


def test_dodgy_parents():
    logger = picologging.Logger('test')
    parent = "potato"
    logger.parent = parent
    with pytest.raises(AttributeError):
        logger.getEffectiveLevel()


def test_add_filter():
    logger = picologging.Logger('test')
    filter = logging.Filter("filter1")
    logger.addFilter(filter)
    assert logger.filters == [filter]
    filter2 = logging.Filter("filter2")
    logger.addFilter(filter2)
    assert logger.filters == [filter, filter2]


def test_remove_filter():
    logger = picologging.Logger('test')
    filter = logging.Filter("filter1")
    logger.addFilter(filter)
    assert logger.filters == [filter]
    logger.removeFilter(filter)
    assert logger.filters == []


def test_no_filter():
    logger = picologging.Logger('test')
    record = picologging.LogRecord('test', logging.INFO, 'test', 1, 'test', (), {})
    assert logger.filter(record) == True


def test_filter_record():
    logger = picologging.Logger('test')
    filter = logging.Filter("hello")
    logger.addFilter(filter)
    record = picologging.LogRecord('hello', logging.INFO, 'test', 1, 'test', (), {})
    record2 = picologging.LogRecord('goodbye', logging.INFO, 'test', 1, 'test', (), {})
    assert logger.filter(record) == True
    assert logger.filter(record2) == False
    logger.removeFilter(filter)
    assert logger.filter(record) == True
    assert logger.filter(record2) == True


def test_filter_callable():
    logger = picologging.Logger('test')
    def filter(record):
        return record.name == 'hello'
    logger.addFilter(filter)
    record = picologging.LogRecord('hello', logging.INFO, 'test', 1, 'test', (), {})
    assert logger.filter(record) == True
    record = picologging.LogRecord('goodbye', logging.INFO, 'test', 1, 'test', (), {})
    assert logger.filter(record) == False


def test_log_debug():
    logger = picologging.Logger('test', logging.DEBUG)
    stream = io.StringIO()
    handler = picologging.StreamHandler(stream)
    handler.setFormatter(picologging.Formatter('%(message)s'))
    logger.addHandler(handler)
    assert logger.debug("Hello World") == None
    result = stream.getvalue()
    assert result == "Hello World\n"

def test_log_debug_info_level_logger():
    logger = picologging.Logger('test', logging.INFO)
    stream = io.StringIO()
    logger.handlers.append(picologging.StreamHandler(stream))
    assert logger.debug("Hello World") == None
    result = stream.getvalue()
    assert result == ""

def test_log_debug_info_level_logger_logging_handler():
    logger = picologging.Logger('test', logging.INFO)
    stream = io.StringIO()
    logger.handlers.append(logging.StreamHandler(stream))
    assert logger.debug("Hello World") == None
    result = stream.getvalue()
    assert result == ""

@pytest.mark.parametrize("level", levels)
def test_log_log(level):
    logger = picologging.Logger('test', level)
    stream = io.StringIO()
    handler = picologging.StreamHandler(stream)
    handler.setFormatter(picologging.Formatter('%(message)s'))
    logger.addHandler(handler)
    assert logger.log(level, "Hello World") == None
    result = stream.getvalue()
    assert result == "Hello World\n"


def test_logger_with_explicit_level(capsys):
    logger = picologging.Logger("test", logging.DEBUG)
    tmp = io.StringIO()
    handler = picologging.StreamHandler(tmp)
    handler.setLevel(logging.DEBUG)
    formatter = picologging.Formatter('%(name)s - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    logger.handlers.append(handler)
    logger.debug("There has been a picologging issue")
    result = tmp.getvalue()
    assert result == "test - DEBUG - There has been a picologging issue\n"
    cap = capsys.readouterr()
    assert cap.out == ""
    assert cap.err == ""



from picologging import Formatter, LogRecord
import logging
import pytest


def test_formatter_default_fmt():
    f = Formatter()
    assert f.datefmt == None
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    s = f.format(record)
    assert s == 'bork bork bork'


def test_formatter_custom_fmt():
    f = Formatter("%(name)s %(levelname)s %(message)s")
    assert f.datefmt == None
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    s = f.format(record)
    assert s == 'hello WARNING bork bork bork'
    assert f.usesTime() == False
    assert f.formatMessage(record) == s


def test_formatter_default_fmt_against_builtin():
    pico_f = Formatter()
    logging_f = logging.Formatter()

    assert pico_f.datefmt == logging_f.datefmt

    pico_record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    logging_record = logging.LogRecord('hello', logging.WARNING, '/serv/', 123, 'bork bork bork', (), None)
    assert pico_f.format(pico_record) == logging_f.format(logging_record)
    assert pico_f.format(pico_record) == logging_f.format(pico_record)
    assert pico_f.formatMessage(pico_record) == logging_f.formatMessage(pico_record)

def test_formatter_custom_datefmt():
    f = Formatter("%(name)s %(levelname)s %(message)s", datefmt="%Y-%m-%d")
    assert f.datefmt == "%Y-%m-%d"
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    s = f.format(record)
    assert s == 'hello WARNING bork bork bork'

def test_formatter_explicit_none_datefmt_style():
    f = Formatter("%(name)s %(levelname)s %(message)s", None, '%')
    assert f.datefmt == None
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    s = f.format(record)
    assert s == 'hello WARNING bork bork bork'

possible_format_strings = [
    "%(message)s",
    "%(message)s is a potato",
    "%(name)s",
    "%(msg)s",
    "%(args)s",
    "%(levelname)s",
    "%(levelno)s",
    "%(pathname)s",
    "%(filename)s",
    "%(module)s",
    "%(funcName)s",
    "%(lineno)d",
    "%(threadName)s",
    "%(process)d",
    "%(processName)s",
    "%(stack_info)s",
    "%(exc_info)s",
    "%(exc_text)s",
]

@pytest.mark.parametrize("field", possible_format_strings)
def test_format_field(field):
    pico_f = Formatter(field)
    log_f = logging.Formatter(field)
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    assert pico_f.format(record) == log_f.format(record)


def test_format_time():
    pico_f = Formatter("%(msecs)d %(relativeCreated)d")
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    assert pico_f.format(record)


def test_asctime_field():
    pico_f = Formatter("%(asctime)s")
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    assert pico_f.format(record)
    assert pico_f.usesTime()


def test_record_with_stack_info():
    pico_f = Formatter("%(message)s")
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None, None, "hello")
    assert pico_f.format(record) == "bork bork bork\nhello"

def test_format_stack():
    pico_f = Formatter("%(message)s")
    assert pico_f.formatStack([1,2,3]) == [1,2,3]

def test_delete_formatter():
    pico_f = Formatter("%(message)s")
    del pico_f


import threading
from picologging import LogRecord
import logging
import pytest
import os

def test_logrecord_standard():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork bork bork', (), None)
    assert record.name == 'hello'
    assert record.msg == 'bork bork bork'
    assert record.levelno == logging.WARNING
    assert record.levelname == 'WARNING'
    assert record.pathname == __file__
    assert record.module == 'test_logrecord'
    assert record.filename == 'test_logrecord.py'
    assert record.args == ()
    assert record.created


def test_logrecord_args():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork %s', ('boom'), None)
    assert record.name == 'hello'
    assert record.msg == 'bork %s'
    assert record.args == ('boom')
    assert record.message == None


def test_logrecord_getmessage_with_args():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork %s', ('boom'), None)
    assert record.message == None
    assert record.getMessage() == "bork boom"
    assert record.message == "bork boom"


def test_logrecord_getmessage_no_args():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork boom', (), None)
    assert record.message == None
    assert record.getMessage() == "bork boom"
    assert record.message == "bork boom"


def test_args_format_mismatch():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork boom %s %s', (0, ), None)
    assert record.message == None
    with pytest.raises(TypeError):
        record.getMessage()


def test_args_len_mismatch():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork boom %s', (0, 1, 2), None)
    assert record.message == None
    with pytest.raises(TypeError):
        record.getMessage()

def test_no_args():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork boom', None, None)
    assert record.message == None
    assert record.getMessage() == "bork boom"
    assert record.message == "bork boom"

def test_no_args_and_format():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork %s', None, None)
    assert record.message == None
    assert record.getMessage() == "bork %s"
    assert record.message == "bork %s"

def test_repr():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork %s', (0, ), None)
    assert repr(record) == f"<LogRecord: hello, 30, {__file__}, 123, 'bork %s'>"

def test_mapping_dict():
    args = {
        "a": "b",
    }
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork %s', (args, ), None)
    assert record.args == {"a": "b"}


def test_threading_info():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork', (), None)
    assert record.thread == threading.get_ident()
    assert record.threadName == None # Not supported


def test_process_info():
    record = LogRecord('hello', logging.WARNING, __file__, 123, 'bork', (), None)
    assert record.process == os.getpid()
    assert record.processName == None # Not supported
